/*
Copyright 2020-2021. Huawei Technologies Co., Ltd. All rights reserved.

Licensed under the Apache License, Version 2.0 (the "License")
you may not use this file except in compliance with the License.
You may obtain a copy of the License at

    https://www.apache.org/licenses/LICENSE-2.0

Unless required by applicable law or agreed to in writing, software
distributed under the License is distributed on an "AS IS" BASIS,
WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
See the License for the specific language governing permissions and
limitations under the License.
*/

package com.huawei.agconnectauth.handlers;

import com.huawei.agconnectauth.AGConnectAuthModule;

import io.flutter.plugin.common.MethodCall;
import io.flutter.plugin.common.MethodChannel.MethodCallHandler;
import io.flutter.plugin.common.MethodChannel.Result;

public class AGConnectAuthMethodCallHandler implements MethodCallHandler {

    private final AGConnectAuthModule agConnectAuthModule;

    public AGConnectAuthMethodCallHandler(AGConnectAuthModule agConnectAuthModule) {
        this.agConnectAuthModule = agConnectAuthModule;
    }

    @Override
    public void onMethodCall(MethodCall call, Result result) {
        switch (call.method) {
            case "signIn":
                agConnectAuthModule.handleSignIn(call, result);
                break;
            case "signInAnonymously":
                agConnectAuthModule.handleSignInAnonymously(result);
                break;
            case "signOut":
                agConnectAuthModule.handleSignOut(result);
                break;
            case "deleteUser":
                agConnectAuthModule.handleDeleteUser(result);
                break;
            case "getCurrentUser":
                agConnectAuthModule.handleGetCurrentUser(result);
                break;
            case "createUserWithEmail":
                agConnectAuthModule.handleCreateUserWithEmail(call, result);
                break;
            case "createUserWithPhone":
                agConnectAuthModule.handleCreateUserWithPhone(call, result);
                break;
            case "resetPasswordWithEmail":
                agConnectAuthModule.handleResetPasswordWithEmail(call, result);
                break;
            case "resetPasswordWithPhone":
                agConnectAuthModule.handleResetPasswordWithPhone(call, result);
                break;
            case "link":
                agConnectAuthModule.handleLink(call, result);
                break;
            case "unlink":
                agConnectAuthModule.handleUnlink(call, result);
                break;
            case "updateEmail":
                agConnectAuthModule.handleUpdateEmail(call, result);
                break;
            case "updatePhone":
                agConnectAuthModule.handleUpdatePhone(call, result);
                break;
            case "updateProfile":
                agConnectAuthModule.handleUpdateProfile(call, result);
                break;
            case "updatePassword":
                agConnectAuthModule.handleUpdatePassword(call, result);
                break;
            case "getUserExtra":
                agConnectAuthModule.handleGetUserExtra(result);
                break;
            case "getToken":
                agConnectAuthModule.handleGetToken(call, result);
                break;
            case "requestEmailVerifyCode":
                agConnectAuthModule.handleRequestEmailVerifyCode(call, result);
                break;
            case "requestPhoneVerifyCode":
                agConnectAuthModule.handleRequestPhoneVerifyCode(call, result);
                break;
            default:
                result.notImplemented();
                break;
        }
    }

}
